@@ -13,12 +13,13 @@
 use v5.14;
 use warnings;
 
-package Protocol::Sys::Virt::URI v10.3.10;
+package Protocol::Sys::Virt::URI v10.3.11;
 
 use parent qw(Exporter);
 
 use Carp qw(croak);
 use Log::Any qw($log);
+use URI::Encode qw(uri_encode uri_decode);
 
 our @EXPORT = qw( parse_url );
 
@@ -27,9 +28,7 @@ sub parse_url {
     my ($base, $query) = split( /\?/, $url, 2 );
     $query //= '';
     my %args = map {
-        s/%([0-9a-z]{2})/chr(hex($1))/gie;
-        # Encode::decode ascii -> Perl-internal????
-        $_;
+        uri_decode($_)
     }
     map {
         my ($key, $val) = split( /=/, $_, 2 );
@@ -42,24 +41,28 @@ sub parse_url {
                 (?<hypervisor>[a-z0-9_]+)
                 (?:\+(?<transport>[a-z0-9_]+))?
                 ://
-                (?:(?<username>[^@]*)
-                   (?:@
-                      (?<password>.*)
-                   )?
-                   :
+                (?:(?<username>[^@]+)@)?
+                ((?<host>[a-z0-9_\-\.]+)
+                 (?:\:(?<port>\d+))?
                 )?
-                (?<host>[a-z0-9_\-\.]+)
                 /
                 (?<type>system|session)
                 $
                 #xi) {
-        my $host = $+{host} // '';
-        my $bare = "$+{hypervisor}:///$type";
+        my $bare = "$+{hypervisor}:///$+{type}";
         $bare .= '?' if ($args{mode} or $args{socket});
-        $bare .= "mode=$args{mode}" if $args{mode};
+        $bare .= 'mode=' . uri_encode($args{mode},
+                                      { encode_reserved => 1 })
+            if $args{mode};
         $bare .= '&' if ($args{mode} and $args{socket});
-        $bare .= "socket=$args{socket}" if $args{socket};
-        return (base => $base, bare => $bare, %+, query => \%args);
+        $bare .= 'socket=' . uri_encode($args{socket},
+                                        { encode_reserved => 1 })
+            if $args{socket};
+        return (base => $base,
+                proxy => $bare,
+                name => "$+{hypervisor}:///$+{type}",
+                %+,
+                query => \%args);
     }
 
     die "Malformed hypervisor URI $url";
@@ -76,7 +79,7 @@ Protocol::Sys::Virt::URI - Helper routines for parsing LibVirt URIs
 
 =head1 VERSION
 
-v10.3.10
+v10.3.11
 
 =head1 SYNOPSIS
 
@@ -92,19 +95,65 @@ Helper functions operating on LibVirt hypervisor URLs.
 
 =head2 parse_url
 
-  my %components = parse_url( 'qemu+ssh://user@pass:hostname/system?param=value&mode=legacy&socket=/path/socket' );
+  my %components = parse_url( 'qemu+ssh://user@hostname:port/system?param=value&mode=legacy&socket=/path/socket' );
   # { base => 'qemu+ssh://user@password:host/system',
-  #   bare => 'qemu:///system?mode=legacy&socket=/path/socket',
+  #   proxy => 'qemu:///system?mode=legacy&socket=/path/socket',
+  #   name  => 'qemu:///system',
   #   query => { param => 'value', mode => 'legacy', socket => '/path/socket' },
   #   transport => 'ssh',
   #   hypervisor => 'qemu',
   #   username => 'user,
-  #   password => 'pass',
   #   host => 'hostname',
+  #   port => 'port',
   # }
 
 Splits the URL into the components necessary for use with the LibVirt API.
 
+=over 8
+
+=item base
+
+This is the URL without the query string and fragment.
+
+=item proxy
+
+This is the URL to be passed through to a proxy: it strips all but
+the parameters applicable to the end point (currently the C<socket>
+and C<mode> parameters are passed through).
+
+=item name
+
+This is the URL as it should be passed to the C<name> parameter in
+the protocol's C<REMOTE_PROC_CONNECT_OPEN> message.
+
+=item query
+
+Contains a hashref with the (URI-decoded) name/value pairs from
+the query string.
+
+=item transport
+
+The fraction of the protocol following the C<+>; eg., C<ssh> in C<qemu+ssh>.
+
+=item hypervisor
+
+The fraction of the protocol before the C<+>, or lacking a plus-sign, the
+protocol; eg., C<qemu> in C<qemu+ssh>.
+
+=item username
+
+The user name embedded in the URL, preceeding the host name, if any.
+
+=item host
+
+The name (or IP address) of the host with the hypervisor.
+
+=item port
+
+The port on the host with the hypervisor to connect to.
+
+=back
+
 =head1 SEE ALSO
 
 L<LibVirt|https://libvirt.org>, L<Sys::Virt>
